//
// System.Runtime.Versioning.ResourceConsumptionAttribute class
//
// Authors
//	Sebastien Pouliot  <sebastien@ximian.com>
//
// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System.Diagnostics;

namespace System.Runtime.Versioning {

	[AttributeUsage (AttributeTargets.Constructor | AttributeTargets.Method | AttributeTargets.Property, Inherited = false)]
	[Conditional ("RESOURCE_ANNOTATION_WORK")]
	public sealed class ResourceConsumptionAttribute: Attribute {

		private ResourceScope resource;
		private ResourceScope consumption;

		public ResourceConsumptionAttribute (ResourceScope resourceScope)
		{
			resource = resourceScope;
			consumption = resourceScope;
		}

		public ResourceConsumptionAttribute (ResourceScope resourceScope, ResourceScope consumptionScope)
		{
			resource = resourceScope;
			consumption = consumptionScope;
		}


		public ResourceScope ConsumptionScope {
			get { return consumption; }
		}

		public ResourceScope ResourceScope {
			get { return resource; }
		}
	}
}
